unit GameViewMain;

interface

uses Classes,
  CastleVectors, CastleComponentSerialize,
  CastleUIControls, CastleControls, CastleKeysMouse, CastleTransform, CastleScene, CastleViewport,  X3DNodes, X3DFields;

type
  TAvatar = class

  private
   type TPersonalia = class
     Gender: string;
     FirstName, LastName, FullName: string;
     KnowPlayer, KnowNPC, KnowNPCProfession: Boolean;
     Age: Integer;
     Profession: string;
     Outfit: string;
     CurrentLocation: string;
     CurrentAction: string;
     GreetingLine: string;
     AnswerLine: string;
     Stamina: Integer;
     Health: Integer;
     Strength: Integer;
     Relationship: Integer;
     Obedience: Integer;
     FollowPlayer, FollowNPC, GoToLocation: boolean;
     ResponseDelay: Integer;
   end;
   var Personalia: TPersonalia;

   Transform: TCastleTransform;
   face_southwest, face_southeast: TCastleScene;

   X,Y, Z, S: single;

   procedure FaceSouthWest(Sender: TObject);
   procedure FaceSouthEast(Sender: TObject);

   procedure ResetPlayer(firstname: string);
   procedure Update(const SecondsPassed: Single; Sender: TObject);

   public

   constructor Create(firstname: string);
   destructor Destroy; override;

end;


type
  { Main view, where most of the application logic takes place. }
  TViewMain = class(TCastleView)
  published
    { Components designed using CGE editor.
      These fields will be automatically initialized at Start. }
    Viewport: TCastleViewport;

    Label1: TCastleLabel;

  private
    Player: TAvatar;
    NPC: TAvatar;

  public
    constructor Create(AOwner: TComponent); override;
    procedure Start; override;
    procedure Update(const SecondsPassed: Single; var HandleInput: Boolean); override;
    function Press(const Event: TInputPressRelease): Boolean; override;
  end;




var
  ViewMain: TViewMain;

implementation

uses SysUtils, CastleRenderOptions;

{ TViewMain ----------------------------------------------------------------- }

constructor TAvatar.Create(firstname: string);
begin

  Personalia := TPersonalia.Create;

  Personalia.FirstName := firstname;

  Transform := TCastleTransform.Create(Transform);

  face_southwest := TCastleScene.Create(face_southwest);
  face_southeast := TCastleScene.Create(face_southeast);

  Transform.Add(face_southwest);
  Transform.Add(face_southeast);

  ViewMain.Viewport.Items.Add(Transform);

  face_southwest.URL:= 'castle-data:/characters/' + Personalia.FirstName + '/' + Personalia.FirstName +'facesw.starling-xml';
  face_southeast.URL:= 'castle-data:/characters/' + Personalia.FirstName + '/' + Personalia.FirstName +'facese.starling-xml';

  ResetPlayer(firstname);

end;

Destructor TAvatar.Destroy;
begin
   FreeAndNil(Personalia.Firstname);
end;

constructor TViewMain.Create(AOwner: TComponent);
begin
  inherited;
  DesignUrl := 'castle-data:/gameviewmain.castle-user-interface';

end;

procedure TViewMain.Start;
var T, V: TVector3;
  begin
  inherited;

    Player := TAvatar.Create('jack');
    Player.face_southwest.Exists := true;
    Player.FaceSouthWest(Player);

    NPC := TAvatar.Create('heather');
    NPC.face_southeast.Exists := true;
    NPC.FaceSouthEast(NPC);

  T := Player.Transform.Translation;
  T.X := 1800;
  T.Y := 420;

  Player.Transform.Translation := T;


  V := NPC.Transform.Translation;
  V.X := 1400;
  V.Y := 420;

  NPC.Transform.Translation := V;

end;

procedure TViewMain.Update(const SecondsPassed: Single; var HandleInput: Boolean);
  begin
  inherited;

  Player.Update(1, Player);
  NPC.Update(1, NPC);


  if Viewport.TransformUnderMouse <> nil then
  begin
    Label1.Caption := Viewport.TransformUnderMouse.Name;
   end
  else
  begin

   Label1.Caption := ' ';
  end;

end;

Procedure TAvatar.Update(const SecondsPassed: Single; Sender: TObject);

 var T: TVector3;
 begin

  T := Transform.Translation;

  X := T.X * SecondsPassed;
  Y := T.Y * SecondsPassed;
  Z := - Transform.WorldBoundingBox.Min.Y + 600;


  S := 1.3 - (0.00200 * T.Y);   // set default size and shrinking factor
  Transform.Scale := Vector3(S, S, S);
  Transform.Translation := T;

end;

procedure TAvatar.ResetPlayer(firstname: string);
begin
  face_southwest.Exists:= false;
  face_southeast.Exists:= false;
end;

procedure TAvatar.FaceSouthWest(Sender: TObject);
begin
  ResetPlayer(Personalia.FirstName);
  face_southwest.Exists:= true;
  face_southwest.PlayAnimation(Personalia.FirstName + 'facesw', true);
end;

procedure TAvatar.FaceSouthEast(Sender: TObject);
begin
  ResetPlayer(Personalia.FirstName);
  face_southeast.Exists:= true;
  face_southeast.PlayAnimation(Personalia.FirstName + 'facese', true);
end;

function TViewMain.Press(const Event: TInputPressRelease): Boolean;
begin

  Result := inherited;
  if Result then Exit;
end;

end.
